﻿/// @file parsewav.h
///
/// parsewav.h
///
#ifndef _PARSEWAV_H_
#define _PARSEWAV_H_

#include <stdio.h>
#include <string.h>
//#define _USE_MATH_DEFINES
//#include <math.h>
#include <vector>
#include "common.h"
#include <wx/wx.h>
#include "parsewav_param.h"
#include "parsewav_buffer.h"
#include "progressbox.h"
#include "rftypebox.h"
#include "maddressbox.h"
#include "errorinfo.h"
#include "parsewav_format.h"
#include "parsewav_util.h"
#include "parsewav_dft.h"

typedef struct pos_st {
	int    start_pos;
	int    end_pos;
	uint32_t start_ms;
	uint32_t end_ms;
} pos_t;

/// @brief 結果レポート用クラス
class REPORT4
{
public:
	REPORT4() {
		memset(save_data_name, 0, sizeof(save_data_name));
		flags = 0;
		baud = 0;
		data_count = 0;
		chksum_err_num = 0;
	}
	~REPORT4() {}

	uint8_t save_data_name[21];
	uint8_t flags;
	uint8_t baud;
	int data_count;
	int chksum_err_num;
	std::vector<pos_t> chksum_err_pos;
};

/// @brief データレコーダの変調波(FSK)を解析して実データに変換するクラス
///
/// waveファイル形式で保存されたデータレコーダの変調波(FSK)1200Hzと2400Hzの波を
/// 解析して搬送波データ、シリアルデータ、実データを取り出す。
/// 逆に実データから変調波のwaveファイルを作成する。
class PARSEWAV
{
public:
	enum enum_file_type {
		UNKNOWN = -1,
		WAV = 0,
		L3C,
		L3B,
		T9X,
		L3,
		REAL,
		PLAIN,
		NO_FILE
	};
private:
	ProgressBox *progbox;
	RfTypeBox *rftypebox;
	MAddressBox *maddressbox;
	wxWindow *parent_window;
	PwErrInfo *errinfo;

	FILE *fio_in;
	FILE *fio_out;
	FILE *fio_outs;

	FILE *fiolog;

	wxString infile;
	wxString outfile;
	wxString outsfile;
	wxString outsfileb;
	wxString outsext;
	wxString logfile;
	wxString *logbuf;

	enum_file_type infile_type;
	enum_file_type outfile_type;

	// input file info
	long long sample_pos;
	long long sample_num;
	long long sample_num_orig;
	uint32_t    sample_ms;

	bool include_header;
	uint8_t save_data_name[21];
	uint8_t rf_header[5];
	uint8_t rf_footer[5];


	PARSEWAV_PARAM param;
	PARSEWAV_UTIL  conv;
	PARSEWAV_DFT   dft;

	int	phase1;
	int phase2;
	int phase2n;
	int phase3;
	int phase4;

//	int buffer_size;
	int phase2_frip;
	int phase2n_baud_count;
	int phase3_baud;

	int baud24_frip;

	int write_pos;
	int half_wave;

	struct parse_carrier_st {
		double x0_prev;
		int    wav_prev;
		int    sample_cnt;
		int    odd;
		int    carr_prev;
	} st_pa_carr;

	struct lamda_st {
		double samples[3];	// 1波のサンプル数 0:1200Hz 1:2400Hz 2:4800Hz
		double us_delta;	// 1サンプルの長さ(us)
		double us_range[3];	// 0:1200Hz 1:2400Hz 2:4800Hz
		double us[3];		// 0:1200Hz 1:2400Hz 2:4800Hz
		double us_min[3];
		double us_max[3];
		double us_avg[3];
		double us_mid[2];	// 0:1800Hz 1:3600Hz
		double us_mid_avg[2];
		double us_limit[2];	// 0:9600Hz 1:19200Hz
	} st_lamda;


	Buf_t<uint8_t> *wave_data;
	Buf_t<uint8_t> *wave_correct_data;
	Buf_t<char>  *carrier_data;
	Buf_t<char>  *serial_data;
	Buf_t<char>  *serial_new_data;
	Buf_t<uint8_t> *binary_data;

	PwErrCode err_num;

	bool chk_wav_mode;
	struct chkwav_st {
		int num;
		int sample_num[2][5];
		int analyze_num;
		int baud_num[2][4];
		int rev_num[2][2];
		double us0avg[2];
		double us1avg[2];
		int amp_max[2];
		int amp_min[2];
	} st_chkwav[2];

	int rep_p1_sample_num[9];
	int rep_p2_error_num;
	int rep_p3_error_num;

	PARSEWAV_FORMAT inwav;
	PARSEWAV_FORMAT outwav;

//	int remain_buf[100];
//	int remain_buf_len;
//	int divide;
//	int surplus;

	wxString buff;
	char cbuff[1000];

	int prev_data_len;
	uint8_t prev_data_body[257];

	std::vector<REPORT4 *> rep4;
	REPORT4 *rep4_itm;

	PwErrType check_wav_format(FILE *fp, wav_header_t *head, wav_fmt_chank_t *fmt, wav_data_chank_t *data);
	PwErrType check_l3c_format(FILE *fp);
	PwErrType check_l3b_format(FILE *fp);
	PwErrType check_t9x_format(FILE *fp);
	PwErrType check_l3_format(FILE *fp);
	PwErrType check_rf_format(FILE *fp, const _TCHAR *in_file, enum_file_type *file_type);
	PwErrType get_first_rf_data(FILE *fp, enum_file_type *file_type);

	int get_wav_sample(int);
	int get_wav_sample(int, int);
	int get_wav_sample(Buf_t<uint8_t> *w_data, int reverse);
	int get_l3c_sample(Buf_t<char> *c_data);
	int get_l3b_sample(Buf_t<char> *s_data);
	int get_t9x_sample(Buf_t<char> *s_data);
	int   get_l3_sample(Buf_t<uint8_t> *b_data);
	uint8_t get_l3_sample();
	uint8_t get_rf_sample();

	void  init_parameters();

	int   decode_to_carrier_wave_bn(int fsk_spd, Buf_t<uint8_t> *w_data, Buf_t<char> *c_data);

	int   find_start_carrier_bit(Buf_t<char> *c_data, Buf_t<char> *s_data, int baud, int *frip);
	int   conv_baud_rate(Buf_t<char> *s_data, Buf_t<char> *sn_data);
	int   find_start_serial_bit(Buf_t<char> *s_data);
	int   parse_carrier_pattern(int fsk_spd, Buf_t<char> *c_data);
	int   decode_to_serial_data(Buf_t<char> *c_data, Buf_t<char> *s_data, int baud, int *frip);
	int   decode_to_binary_data(Buf_t<char> *s_data, Buf_t<uint8_t> *b_data);
	int   encode_to_serial_data(uint8_t bin_data, Buf_t<char> *s_data);
	int	  encode_to_carrier_data(Buf_t<char> *s_data, Buf_t<char> *c_data);
	int   encode_to_wave_data(Buf_t<char> *c_data, uint8_t *w_data);

	int   decode_phase1(int fsk_spd, Buf_t<uint8_t> *w_data, Buf_t<uint8_t> *wc_data, Buf_t<char> *c_data, Buf_t<char> *s_data, Buf_t<char> *sn_data, Buf_t<uint8_t> *b_data, int start_phase, int corret_type, int reverse);
	int   decode_phase1(int fsk_spd, Buf_t<uint8_t> *w_data, Buf_t<uint8_t> *wc_data, Buf_t<char> *c_data, Buf_t<char> *s_data, Buf_t<char> *sn_data, Buf_t<uint8_t> *b_data, int start_phase);
	int   decode_phase2(int fsk_spd, Buf_t<char> *c_data, Buf_t<char> *s_data, Buf_t<char> *sn_data, Buf_t<uint8_t> *b_data, int start_phase);
	int   decode_phase2n(Buf_t<char> *s_data, Buf_t<char> *sn_data, Buf_t<uint8_t> *b_data, int start_phase);
	int   decode_phase3(Buf_t<char> *s_data, Buf_t<uint8_t> *b_data, int start_phase);
	int	  decode_phase4(Buf_t<uint8_t> *b_data);


	int   encode_phase4(Buf_t<uint8_t> *b_data, Buf_t<char> *s_data, Buf_t<char> *c_data);
	int   encode_phase3(Buf_t<uint8_t> *b_data, Buf_t<char> *s_data, Buf_t<char> *c_data, int start_phase);
	int   encode_phase2(Buf_t<char> *s_data, Buf_t<char> *c_data, int start_phase, bool end_data);
	int   encode_phase1(Buf_t<char> *c_data, bool end_data);

	void  reporting();
	void  reporting_analyze();
	void  write_log(const wxString &, int);

	bool check_extension(const wxString &filename, const wxString &ext);

	int get_sample_rate(int);
//	int out_sample_buffer(int16_t *new_buf, size_t new_len, size_t size, FILE *fp);

public:
	PARSEWAV(wxWindow *parent);
	~PARSEWAV();

	bool OpenDataFile(wxString &);
	void CloseDataFile();
	PwErrType CheckFileFormat(FILE *fp, const _TCHAR *in_file, enum_file_type *file_type);
	PwErrType SeekFileFormat(FILE *fp, const _TCHAR *in_file, enum_file_type file_type);

	bool OpenOutFile(wxString &);
	void CloseOutFile();

	int InitFileHeader(FILE *, enum_file_type file_type);
	void SetFileHeader(FILE *, enum_file_type in_file_type, enum_file_type out_file_type);
//	void OutConvSampleData(void *, uint32_t, size_t, size_t, FILE *);

	bool ExportData(enum_file_type file_type = UNKNOWN);
	PwErrType DecodeData();
	PwErrType EncodeData();
	int AnalyzeWave();

	void GetFileNameBase(wxString &);

	void SetLogBufferPtr(wxString *);

	bool ShowRfTypeBox();
	bool ShowMAddressBox(bool);


	bool IsOpenedDataFile() { return (fio_in != NULL ? true : false); }
	enum_file_type GetDataFileType() { return infile_type; }
	int GetRfDataFormat();

	PARSEWAV_PARAM* GetParam() { return &param; }
	void SetParam(PARSEWAV_PARAM* data) { param = *data; }

	void SetSampleRate(int value) { param.SetSampleRate(value); }
	void SetSampleBits(int value) {	param.SetSampleBits(value); }
	void SetBaud(int value)		{ param.SetBaud(value); }
	void SetAutoBaud(int value) { param.SetAutoBaud(value); }
	void SetReverseWave(int value) { param.SetReverseWave(value); }
	void SetHalfWave(int value) { param.SetHalfWave(value); }
	void SetCorrectWave(int value) { param.SetCorrectWave(value); }
	void SetCorrectType(int value) { param.SetCorrectType(value); }
	void SetFskSpeed(int value) { param.SetFskSpeed(value); }
	void SetFreq(int num, int value) { param.SetFreq(num, value); }
	void SetFrequency(int magnify);
	void SetRange(int num, int value) { param.SetRange(num, value); }
	void SetWordSelect(int value) { param.SetWordSelect(value); }
	void SetChangeGapSize(int value) { param.SetChangeGapSize(value); }
	void SetDebugMode(int value) { param.SetDebugMode(value); }

	int GetSampleRate(void) { return param.GetSampleRate(); }
	int GetSampleBits(void) { return param.GetSampleBits(); }
	int GetBaud(void)		{ return param.GetBaud(); }
	int	GetAutoBaud(void)   { return param.GetAutoBaud(); }
	int GetReverseWave(void) { return param.GetReverseWave(); }
	int GetHalfWave(void)	{ return param.GetHalfWave(); }
	int GetCorrectWave(void) { return param.GetCorrectWave(); }
	int GetCorrectType(void) { return param.GetCorrectType(); }
	int GetFskSpeed(void) { return param.GetFskSpeed(); }
	int GetBaseFreq(void) { return param.GetBaseFreq(); }
	int GetFreq(int num) { return param.GetFreq(num); }
	int	GetRange(int num) { return param.GetRange(num); }
	int	GetWordSelect(void) { return param.GetWordSelect(); }
	int	GetChangeGapSize(void) { return param.GetChangeGapSize(); }
	int GetDebugMode(void) { return param.GetDebugMode(); }
};

#endif
