/// @file parsewav_format.cpp
///
/// @brief wav format
///
/// @author Sasaji
/// @date   2017.12.01
///

#include <string.h>
#include "parsewav_format.h"

PARSEWAV_FORMAT::PARSEWAV_FORMAT()
{
	memset(&head, 0, sizeof(head));
	memset(&fmt, 0, sizeof(fmt));
	memset(&data, 0, sizeof(data));
}

PARSEWAV_FORMAT::PARSEWAV_FORMAT(int rate, int bits, int channels)
{
	this->Init(rate, bits, channels);
}

PARSEWAV_FORMAT::~PARSEWAV_FORMAT()
{
}

void PARSEWAV_FORMAT::Init(int rate, int bits, int channels)
{
	// wav file header
	memcpy(head.RIFF, "RIFF", 4);
	head.file_len = 0;
	memcpy(head.WAVE, "WAVE", 4);

	memcpy(fmt.fmt, "fmt ", 4);
	fmt.format_id = 1;	// Linear PCM
	fmt.channels = (uint16_t)channels;
	fmt.sample_rate = (uint32_t)rate;
	fmt.data_speed = (uint32_t)(rate * channels);
	fmt.block_size = (uint16_t)(bits * channels / 8);
	fmt.sample_bits = bits;
	fmt.fmt_size = 16;

	memcpy(data.data, "data", 4);
	data.data_len = 0;
}

int PARSEWAV_FORMAT::Out(uint8_t *buffer)
{
	int len = 0;
	int len_all = 0;
	len = sizeof(head);
	memcpy(buffer + len_all, &head, len);
	len_all += len;
	len = sizeof(fmt);
	memcpy(buffer + len_all, &fmt, len);
	len_all += len;
	len = sizeof(data);
	memcpy(buffer + len_all, &data, len);
	len_all += len;

	return len_all;
}

int PARSEWAV_FORMAT::Out(FILE *fp)
{
	int len = 0;
	int len_all = 0;
	len = sizeof(head);
	len_all += len;
	fwrite((void *)&head, sizeof(uint8_t), len, fp);
	len = sizeof(fmt);
	len_all += len;
	fwrite((void *)&fmt, sizeof(uint8_t), len, fp);
	len = sizeof(data);
	len_all += len;
	fwrite((void *)&data, sizeof(uint8_t), len, fp);

	return len_all;
}

void PARSEWAV_FORMAT::OutSize(FILE *fp)
{
	uint32_t file_size = 0;
	uint32_t riff_size = 0;
	uint32_t data_size = 0;

	fseek(fp, 0, SEEK_END);
	file_size = (uint32_t)ftell(fp);

	riff_size = file_size - 8;

	fseek(fp, 4, SEEK_SET);
	fwrite((uint32_t *)&riff_size, sizeof(uint32_t), 1, fp);

	data_size = file_size - 44;

	fseek(fp, 40, SEEK_SET);
	fwrite((uint32_t *)&data_size, sizeof(uint32_t), 1, fp);
}
